@@ -30,8 +30,8 @@ column_to_rownames <- function(.data, var = "rowname"){
   df <-
     as.data.frame(.data)  |>
     remove_rownames()
-  if(!var %in% colnames(df)){
-    stop("Variable '", var, "' not in data.", call. = FALSE)
+  if (!var %in% colnames(df)) {
+    cli::cli_abort("Variable {.val {var}} not found in data (columns are: {.val {colnames(df)}}).")
   }
   rownames(df) <- df[[var]]
   df[[var]] <- NULL
@@ -42,7 +42,7 @@ column_to_rownames <- function(.data, var = "rowname"){
 rownames_to_column <-  function(.data, var = "rowname"){
   col_names <- colnames(.data)
   if (var %in% col_names) {
-    stop("Column `", var, "` already exists in `.data`.")
+    cli::cli_abort("Column {.field {var}} already exists in {.obj .data}.")
   }
   .data[, var] <- rownames(.data)
   rownames(.data) <- NULL
@@ -237,10 +237,10 @@ get_biplot <- function(x,
                        show_unit_circle = TRUE,
                        expand = NULL){
   if(!show %in% c("both", "var", "ind")){
-    stop("`show` must be one of 'var', 'ind', or 'both'", call. = FALSE)
+    cli::cli_abort("Invalid value for {.arg show}. Must be one of 'var', 'ind', or 'both'.")
   }
   if(!inherits(x, "pca")){
-    stop("`x` must be an object computed with 'pca()'", call. = FALSE)
+    cli::cli_abort("Invalid object type for {.arg x}. Must be an object computed with {.fn pca()}.")
   }
   var <- x$var$coord
   ind <- x$ind$coord
@@ -390,10 +390,10 @@ plot.pca <- function(x,
                      axis = 1,
                      ...){
   if(!which %in% c("contrib", "cos2", "coord")){
-    stop("`show` must be one of 'contrib', 'cos2', or 'coord'", call. = FALSE)
+    cli::cli_abort("Invalid value for {.arg show}. Must be one of 'contrib', 'cos2', or 'coord'.")
   }
   if(!type %in% c("var", "ind", "biplot")){
-    stop("`show` must be one of 'var', 'ind', or 'biplot'", call. = FALSE)
+    cli::cli_abort("{.arg show} must be one of 'var', 'ind', or 'biplot'.")
   }
   rotate_x <- function(data, column_to_plot, labels_vec, rot_angle, ...) {
     data <- data[order(data[,column_to_plot], decreasing = TRUE), ]
@@ -498,42 +498,53 @@ run_progress <- function(pb,
 }
 
 check_names_dir <- function(name, names_dir, dir){
-  if(!name %in% names_dir){
-    stop(paste("'", name, "' not found in '",
-               paste(getwd(), sub(".", "", dir), sep = ""), "'", sep = ""),
-         call. = FALSE)
+  if (!name %in% names_dir) {
+    cli::cli_abort("{.val {name}} not found in {.path {file.path(getwd(), sub('.', '', dir))}}.")
   }
 }
 
 check_ebi <- function() {
   if (!requireNamespace("EBImage", quietly = TRUE)) {
+
     if (interactive()) {
-      inst <- switch(menu(c("Yes", "No"),
-                          title = "Package {EBImage} is required but not installed.\nDo you want to install it now?"),
-                     "yes", "no")
+      inst <- switch(
+        menu(c("Yes", "No"),
+             title = cli::cli_alert_danger(
+               "Package {.pkg EBImage} is required but not installed.\nDo you want to install it now?")),
+        "yes", "no"
+      )
+
       if (inst == "yes") {
         if (!requireNamespace("BiocManager", quietly = TRUE)) {
+          cli::cli_alert_info("Installing {.pkg BiocManager}...")
           install.packages("BiocManager", quiet = TRUE)
         }
+
+        cli::cli_alert_info("Installing {.pkg EBImage} from Bioconductor...")
         BiocManager::install("EBImage", update = FALSE, ask = FALSE, quiet = TRUE)
+
         if (!requireNamespace("EBImage", quietly = TRUE)) {
-          message("Installation of {EBImage} failed. Please install it manually.")
+          cli::cli_alert_danger("Installation of {.pkg EBImage} failed. Please install it manually.")
           return(FALSE)
         }
+
       } else {
-        message("To use {pliman}, please install {EBImage} following the directions at 'https://bioconductor.org/packages/EBImage'")
+        cli::cli_alert_warning("To use {.pkg pliman}, please install {.pkg EBImage} from {.url https://bioconductor.org/packages/EBImage}")
         return(FALSE)
       }
+
     } else {
-      message("Package {EBImage} is required. Please install it following the directions at 'https://bioconductor.org/packages/EBImage'")
+      cli::cli_alert_danger("Package {.pkg EBImage} is required. Please install it from {.url https://bioconductor.org/packages/EBImage}")
       return(FALSE)
     }
   }
+
   return(TRUE)
 }
 
 
 
+
 # correct coordinates in analyze_objects_shp()
 correct_coords <- function(coords, nrowimg, ncolimg, nrow, ncol){
   get_row_number <- function(vector, rows, cols) {
@@ -559,29 +570,33 @@ correct_coords <- function(coords, nrowimg, ncolimg, nrow, ncol){
 
 check_mapview <- function() {
   packages <- c("mapview", "mapedit", "leaflet", "leafem")
-  mapv <- !requireNamespace("mapview", quietly = TRUE)
-  mape <- !requireNamespace("mapedit", quietly = TRUE)
-  leafl <- !requireNamespace("leaflet", quietly = TRUE)
-  leafe <- !requireNamespace("leafem", quietly = TRUE)
-  missing_packages <- packages[c(mapv, mape, leafl, leafe)]
-  if (length(missing_packages) > 0) {
+  missing <- packages[!vapply(packages, requireNamespace, logical(1), quietly = TRUE)]
+
+  if (length(missing) > 0) {
     if (interactive()) {
-      inst <- switch(menu(c("Yes", "No"),
-                          title = paste("Packages", paste(missing_packages, collapse = ", "),
-                                        "are required to use the `viewer = 'mapview' option`.\nDo you want to install them now?")),
-                     "yes", "no")
+      msg <- cli::cli_alert_danger(
+        "The following packages are required to use {.arg viewer = 'mapview'}: {.pkg {missing}}"
+      )
+      inst <- switch(menu(
+        c("Yes", "No"),
+        title = paste0(msg, "\nDo you want to install them now?")
+      ), "yes", "no")
+
       if (inst == "yes") {
-        install.packages(missing_packages, quiet = TRUE)
+        cli::cli_alert_info("Installing packages: {.pkg {missing}}")
+        install.packages(missing, quiet = TRUE)
       } else {
-        message("To use viewer = 'mapview', first install the required packages:", paste(missing_packages, collapse = ", "))
+        cli::cli_alert_warning("To use {.arg viewer = 'mapview'}, please install: {.pkg {missing}}")
       }
+
     } else {
-      message("To use viewer = 'mapview', first install the required packages:", paste(missing_packages, collapse = ", "))
+      cli::cli_alert_warning("To use {.arg viewer = 'mapview'}, install the following packages: {.pkg {missing}}")
     }
   }
 }
 
 
+
 # get RGB values from a mask computed with EBImage::watershed()
 get_rgb <- function(img, data_mask, index){
   data.frame(object = index,
@@ -595,9 +610,13 @@ check_inf <- function(data){
   indx <- apply(data, 2, function(x){
     any(is.na(x) | is.infinite(x))
   })
-  if(any(indx) == TRUE){
-    warning("Columns ", paste(colnames(data[indx]), collapse = ", "), " with infinite/NA values removed.", call. = FALSE)
+  if (any(indx)) {
+    cli::cli_warn(c(
+      "!" = "The following columns contain {.val Inf} or {.val NA} values and were removed:",
+      " " = "{.val {colnames(data[indx])}}"
+    ))
   }
+
   data[,colnames(data[!indx])]
 }
 
@@ -705,103 +724,152 @@ ggplot_color <- function(n = 1){
 #' set_wd_here()
 #' open_wd_here()
 #' }
-set_wd_here <- function(path = NULL){
-  if(!requireNamespace("rstudioapi", quietly = TRUE)) {
-    if(interactive() == TRUE){
-      inst <-
-        switch(menu(c("Yes", "No"), title = "Package {rstudioapi} required but not installed.\nDo you want to install it now?"),
-               "yes", "no")
-      if(inst == "yes"){
+set_wd_here <- function(path = NULL) {
+  if (!requireNamespace("rstudioapi", quietly = TRUE)) {
+    if (interactive()) {
+      inst <- switch(menu(c("Yes", "No"),
+                          title = cli::cli_alert_danger(
+                            "Package {.pkg rstudioapi} is required but not installed.\nDo you want to install it now?"
+                          )),
+                     "yes", "no")
+
+      if (inst == "yes") {
+        cli::cli_alert_info("Installing {.pkg rstudioapi}...")
         install.packages("rstudioapi", quiet = TRUE)
-      } else{
-        message("To use `set_wd_here()`, first install {rstudioapi}.")
+      } else {
+        cli::cli_alert_warning("To use {.fn set_wd_here}, please install {.pkg rstudioapi}.")
+        return(invisible(NULL))
       }
+    } else {
+      cli::cli_alert_warning("To use {.fn set_wd_here}, please install {.pkg rstudioapi}.")
+      return(invisible(NULL))
     }
-  } else{
-    dir_path <- dirname(rstudioapi::documentPath())
-    if(!is.null(path)){
-      dir_path <- paste0(dir_path, "/", path)
-    }
-    d <- try(setwd(dir_path), TRUE)
-    if(inherits(d, "try-error")){
-      cat(paste0("Cannot change working directory to '", dir_path, "'."))
+  }
+
+  dir_path <- dirname(rstudioapi::documentPath())
+
+  if (!is.null(path)) {
+    dir_path <- file.path(dir_path, path)
+  }
+
+  d <- try(setwd(dir_path), silent = TRUE)
+
+  if (inherits(d, "try-error")) {
+    cli::cli_alert_danger("Cannot change working directory to {.path {dir_path}}.")
+    if (interactive()) {
       done <- readline(prompt = "Do you want to create this folder now? (y/n) ")
-      if(done == "y"){
-        dir.create(dir_path)
-        message("Directory '", dir_path, "' created.")
+      if (tolower(done) == "y") {
+        dir.create(dir_path, recursive = TRUE)
+        cli::cli_alert_success("Directory {.path {dir_path}} created.")
         setwd(dir_path)
-        message("Working directory set to '", dir_path, "'")
+        cli::cli_alert_success("Working directory set to {.path {dir_path}}.")
       }
-    } else{
-      message("Working directory set to '", dir_path, "'")
     }
+  } else {
+    cli::cli_alert_success("Working directory set to {.path {dir_path}}.")
   }
+
+  invisible(getwd())
 }
 
 #' @export
 #' @name utils_wd
-get_wd_here <- function(path = NULL){
-  if(!requireNamespace("rstudioapi", quietly = TRUE)) {
-    if(interactive() == TRUE){
-      inst <-
-        switch(menu(c("Yes", "No"), title = "Package {rstudioapi} required but not installed.\nDo you want to install it now?"),
-               "yes", "no")
-      if(inst == "yes"){
+get_wd_here <- function(path = NULL) {
+  if (!requireNamespace("rstudioapi", quietly = TRUE)) {
+    if (interactive()) {
+      inst <- switch(menu(c("Yes", "No"),
+                          title = cli::cli_alert_danger(
+                            "Package {.pkg rstudioapi} is required but not installed.\nDo you want to install it now?"
+                          )),
+                     "yes", "no")
+      if (inst == "yes") {
+        cli::cli_alert_info("Installing {.pkg rstudioapi}...")
         install.packages("rstudioapi", quiet = TRUE)
-      } else{
-        message("To use `get_wd_here()`, first install {rstudioapi}.")
+      } else {
+        cli::cli_alert_warning("To use {.fn get_wd_here}, please install {.pkg rstudioapi}.")
+        return(invisible(NULL))
       }
+    } else {
+      cli::cli_alert_warning("To use {.fn get_wd_here}, please install {.pkg rstudioapi}.")
+      return(invisible(NULL))
     }
-  } else{
-    dir_path <- dirname(rstudioapi::documentPath())
-    if(!is.null(path)){
-      dir_path <- paste0(dir_path, "/", path)
-    }
-    dir_path
   }
-}
-#' @export
-#' @name utils_wd
-open_wd_here <- function(path = get_wd_here()){
-  if(!requireNamespace("utils", quietly = TRUE)) {
-    if(interactive() == TRUE){
-      inst <-
-        switch(menu(c("Yes", "No"), title = "Package {utils} required but not installed.\nDo you want to install it now?"),
-               "yes", "no")
-      if(inst == "yes"){
-        install.packages("utils", quiet = TRUE)
-      } else{
-        message("To use `open_wd_here()`, first install {utils}.")
-      }
-    }
-  } else{
-    utils::browseURL(url = path)
-  }
-}
 
+  dir_path <- dirname(rstudioapi::documentPath())
+  if (!is.null(path)) {
+    dir_path <- file.path(dir_path, path)
+  }
+  return(dir_path)
+}
 #' @export
 #' @name utils_wd
-open_wd <- function(path = getwd()){
-  if(!requireNamespace("utils", quietly = TRUE)) {
-    if(interactive() == TRUE){
-      inst <-
-        switch(menu(c("Yes", "No"), title = "Package {utils} required but not installed.\nDo you want to install it now?"),
-               "yes", "no")
-      if(inst == "yes"){
+open_wd_here <- function(path = get_wd_here()) {
+  if (!requireNamespace("utils", quietly = TRUE)) {
+    if (interactive()) {
+      inst <- switch(menu(c("Yes", "No"),
+                          title = cli::cli_alert_danger(
+                            "Package {.pkg utils} is required but not installed.\nDo you want to install it now?"
+                          )),
+                     "yes", "no")
+      if (inst == "yes") {
+        cli::cli_alert_info("Installing {.pkg utils}...")
         install.packages("utils", quiet = TRUE)
-      } else{
-        message("To use `open_wd()`, first install {utils}.")
+      } else {
+        cli::cli_alert_warning("To use {.fn open_wd_here}, please install {.pkg utils}.")
+        return(invisible(NULL))
       }
+    } else {
+      cli::cli_alert_warning("To use {.fn open_wd_here}, please install {.pkg utils}.")
+      return(invisible(NULL))
     }
-  } else{
-    utils::browseURL(url = path)
   }
+
+  cli::cli_alert_info("Opening directory: {.path {path}}")
+  utils::browseURL(url = path)
+}
+#' @export
+#' @name utils_wd
+open_wd <- function(path = getwd()) {
+  if (!requireNamespace("utils", quietly = TRUE)) {
+    if (interactive()) {
+      inst <- switch(menu(c("Yes", "No"),
+                          title = cli::cli_alert_danger(
+                            "Package {.pkg utils} is required but not installed.\nDo you want to install it now?"
+                          )),
+                     "yes", "no")
+      if (inst == "yes") {
+        cli::cli_alert_info("Installing {.pkg utils}...")
+        install.packages("utils", quiet = TRUE)
+      } else {
+        cli::cli_alert_warning("To use {.fn open_wd}, please install {.pkg utils}.")
+        return(invisible(NULL))
+      }
+    } else {
+      cli::cli_alert_warning("To use {.fn open_wd}, please install {.pkg utils}.")
+      return(invisible(NULL))
+    }
+  }
+
+  cli::cli_alert_info("Opening directory: {.path {path}}")
+  utils::browseURL(url = path)
 }
 
 
 parse_formula <- function(formula, valid_indices) {
   eval(parse(text = sprintf("function(%s) %s", paste0(toupper(names(valid_indices)), collapse = ", "), formula)))
 }
+used_layers <- function(indices, formula) {
+  formula_lower <- tolower(formula)
+  vars_extracted <- unique(unlist(regmatches(formula_lower, gregexpr("[a-z]+", formula_lower))))
+  used_names <- names(indices)[names(indices) %in% vars_extracted & !is.na(indices)]
+  as.numeric(indices[used_names])
+}
+unavailable_layers <- function(indices, formula) {
+  formula_lower <- tolower(formula)
+  vars_extracted <- unique(unlist(regmatches(formula_lower, gregexpr("[a-z]+", formula_lower))))
+  used_names <- names(indices)[names(indices) %in% vars_extracted & is.na(indices)]
+  which(names(indices) %in% used_names)
+}
 compute_outsize <- function(pct) {
   if (length(pct) == 1) {
     pct <- rep(pct, 2)
@@ -829,35 +897,235 @@ add_missing_columns <- function(data) {
 }
 
 check_and_install_package <- function(pkg, reason = NULL) {
-  # Reason is optional and can describe why the package is needed
   if (!requireNamespace(pkg, quietly = TRUE)) {
 
-    # Construct the message for the package requirement
-    package_message <- paste0("Package {", pkg, "} is required", if (!is.null(reason)) paste0(" to ", reason), " but is not available.")
+    # Mensagem personalizada com ou sem razão
+    pkg_msg <- if (!is.null(reason)) {
+      cli::cli_alert_danger("Package {.pkg {pkg}} is required to {reason}, but it is not installed.")
+    } else {
+      cli::cli_alert_danger("Package {.pkg {pkg}} is required but not installed.")
+    }
 
     if (interactive()) {
-      # Interactive mode: ask the user if they want to install the package
       inst <- menu(
         choices = c("Yes", "No"),
-        title = paste0(package_message, "\nDo you want to install it now?")
+        title = paste0("Do you want to install {.pkg ", pkg, "} now?")
       )
 
-      if (inst == 1) {  # If the user selects "Yes"
-        message("Installing '", pkg, "' package...")
+      if (inst == 1) {
+        cli::cli_alert_info("Installing {.pkg {pkg}}...")
         install.packages(pkg, quiet = TRUE)
 
         if (requireNamespace(pkg, quietly = TRUE)) {
-          message("Package '", pkg, "' successfully installed.")
+          cli::cli_alert_success("Package {.pkg {pkg}} successfully installed.")
         } else {
-          stop("Package '", pkg, "' installation failed. Please try again.")
+          cli::cli_abort("Installation of {.pkg {pkg}} failed. Please try again manually.")
         }
+
       } else {
-        message("You chose not to install '", pkg, "'. The feature depending on this package will not be available.")
+        cli::cli_alert_warning("You chose not to install {.pkg {pkg}}. The feature requiring it will not be available.")
       }
 
     } else {
-      # Non-interactive mode: stop with an error message
-      stop(package_message, " Please install it manually: install.packages('", pkg, "')")
+      # Modo não interativo: erro direto
+      cli::cli_abort(c(
+        "!" = "Package {.pkg {pkg}} is required{if (!is.null(reason)) paste0(' to ', reason)}.",
+        "i" = "Please install it manually with {.code install.packages('{pkg}')}"
+      ))
     }
   }
 }
+
+
+#' Generate Version 7 UUIDs or Random UUIDs
+#'
+#' This function generates one or more UUIDs (Universally Unique Identifiers).
+#' By default, it generates Version 7 UUIDs, which are time-ordered and suitable
+#' for use cases requiring efficient indexing and sorting by creation time.
+#' Alternatively, random Version 4 UUIDs can be generated by setting `usetime = FALSE`.
+#'
+#' @param n Integer. Number of UUIDs to generate. Default is `1`.
+#' @param uppercase Logical. If `TRUE`, the generated UUIDs are returned in uppercase letters. Default is `FALSE`.
+#' @param usetime Logical. If `TRUE`, generates Version 7 UUIDs using the current timestamp.
+#' If `FALSE`, generates random Version 4 UUIDs. Default is `FALSE`.
+#'
+#' @details
+#' - **Version 7 UUIDs**: These are time-ordered UUIDs based on the current timestamp in milliseconds
+#' since the Unix epoch (`1970-01-01 00:00:00 UTC`). They are ideal for scenarios requiring
+#' chronological sorting or indexing.
+#' - **Version 4 UUIDs**: These are randomly generated UUIDs that do not depend on time,
+#' ensuring uniqueness through random hexadecimal values.
+#'
+#' @return A character vector of UUIDs of length `n`.
+#'
+#' @export
+#' @examples
+#' library(pliman)
+#' # Generate a single UUID
+#' uuid()
+#'
+#' # Generate 5 UUIDs in uppercase
+#' uuid(n = 3, uppercase = TRUE)
+#'
+#' # Generate two random UUIDs
+#' uuid(n = 2, usetime = FALSE)
+#'
+uuid <- function(n = 1,
+                 uppercase = FALSE,
+                 usetime = FALSE){
+  uuidhelp <- function(){
+    hex_digits <- c(0:9, letters[1:6])
+
+    uuid <- paste0(
+      paste0(sample(hex_digits, 8, replace = TRUE), collapse = ""),
+      "-",
+      paste0(sample(hex_digits, 4, replace = TRUE), collapse = ""),
+      "-",
+      "4", paste0(sample(hex_digits, 3, replace = TRUE), collapse = ""), # Version 4 UUID
+      "-",
+      sample(c("8", "9", "a", "b"), 1), paste0(sample(hex_digits, 3, replace = TRUE), collapse = ""), # Variant
+      "-",
+      paste0(sample(hex_digits, 12, replace = TRUE), collapse = "")
+    )
+  }
+
+  if(usetime){
+    uuids <- replicate(n, uuid_v7())
+  } else{
+    uuids <- replicate(n, uuidhelp())
+  }
+
+  if(uppercase){
+    uuids <- toupper(uuids)
+  }
+  return(uuids)
+}
+
+
+
+#' Compute Shannon Entropy
+#'
+#' This function calculates the Shannon entropy of a numeric vector.
+#'
+#' @param x A numeric vector containing the values for which entropy will be computed.
+#'
+#' @return A numeric value representing the entropy.
+#' @export
+#'
+#' @examples
+#' library(pliman)
+#' x <- c(1, 2, 2, 3, 3, 3, 4, 4, 4, 4)
+#' entropy(x)
+entropy <- function(x){
+  helper_entropy(x)
+}
+
+#' Extract UUID from filenames
+#'
+#' This function extracts a UUID (Universal Unique Identifier) from the
+#' filename, using a regular expression that specifically identifies the
+#' standard UUID format.
+#'
+#' @param filename A character vector containing filenames or strings
+#' @return A character vector with extracted UUIDs (or NA if none found)
+#' @export
+#' @examples
+#' library(pliman)
+#' file <- "Grãos - contagem_f68bca60-c8cf-4272-9448-3f28891a97cd.jpg"
+#' file2 <- "Grãos - contagem_f68bca60-c8cf-4272-9448-3f8891a97cd.jpg"
+#' get_uuid(file)
+get_uuid <- function(filename) {
+  patterns <- "[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}"
+  uuidtext <- regmatches(filename, regexpr(patterns, filename))
+  texts <- unname(
+    sapply(filename, function(x){
+      val <- regmatches(x, regexpr(patterns, x))
+      if(length(val) > 0){
+        val
+      } else{
+        NA
+      }
+    })
+  )
+  no_uuid_idx <- which(is.na(texts))
+  if (length(no_uuid_idx) > 0) {
+    cli::cli_warn("No UUID found in {length(no_uuid_idx)} entr{?y/ies} ({.val {filename[no_uuid_idx]}})")
+  }
+  return(texts)
+}
+
+
+#' Lin's Concordance Correlation Coefficient (CCC)
+#'
+#' Computes Lin's Concordance Correlation Coefficient (CCC) between observed and
+#' predicted values. Also returns Pearson's correlation coefficient and root
+#' mean squared error (RMSE). If the input is a grouped data frame
+#' (`grouped_df`), the function will return results for each group.
+#'
+#' The CCC is defined as:
+#'
+#' \deqn{\rho_c = \frac{2 \cdot \text{Cov}(x, y)}{\text{Var}(x) + \text{Var}(y) + (\bar{x} - \bar{y})^2}}
+#'
+#' where:
+#' * \eqn{\text{Cov}(x, y)} is the covariance between observed and predicted values
+#' * \eqn{\text{Var}(x)} and \eqn{\text{Var}(y)} are the variances of the observed and predicted values
+#' * \eqn{\bar{x}} and \eqn{\bar{y}} are the means of the observed and predicted values
+#'
+#' @param data A data frame containing the columns for observed and predicted values.
+#' @param real The column name (unquoted) corresponding to the observed values.
+#' @param predito The column name (unquoted) corresponding to the predicted values.
+#'
+#' @return A data frame with the following columns:
+#' * `r`: Pearson correlation coefficient
+#' * `ccc`: Lin's Concordance Correlation Coefficient
+#' * `rmse`: Root mean squared error
+#'
+#' @export
+#' @examples
+#' library(dplyr)
+#' library(pliman)
+#' df <- data.frame(
+#'   group = rep(c("A", "B"), each = 5),
+#'   real = c(1:5, 2:6),
+#'   predicted = c(1.1, 2, 2.9, 4.1, 5, 2.2, 3.1, 4, 4.8, 6.1)
+#' )
+#'
+#' # Without grouping
+#' ccc(df, real, predicted)
+#'
+#' # With grouping
+#' df |>
+#'   group_by(group) |>
+#'   ccc(real, predicted)
+#'
+ccc <- function(data, real, predito) {
+  if(dplyr::is.grouped_df(data)){
+    data |>
+      dplyr::group_modify(~ccc(.x, {{real}}, {{predito}})) |>
+      dplyr::ungroup()
+  } else{
+    x <- data |> dplyr::select({{real}}) |> dplyr::pull()
+    y <- data |> dplyr::select({{predito}}) |> dplyr::pull()
+    mean_x <- mean(x)
+    mean_y <- mean(y)
+    var_x  <- var(x)
+    var_y  <- var(y)
+    cov_xy <- cov(x, y)
+    r <- cov_xy / (sqrt(var_x) * sqrt(var_y))
+    rmse <- sqrt(mean((x - y)^2))
+    ccc <- (2 * cov_xy) / (var_x + var_y + (mean_x - mean_y)^2)
+    return(data.frame(r = r, ccc = ccc,  rmse = rmse))
+  }
+}
+
+trunc_path <- function(path, max_chars = 100, sep = "...") {
+  n <- nchar(path)
+  if (n <= max_chars) return(path)
+  # allocate half the remaining length to prefix/suffix
+  half <- floor((max_chars - nchar(sep)) / 2)
+  paste0(
+    substr(path, 1, half),
+    sep,
+    substr(path, n - half + 1, n)
+  )
+}
